@@ -19,6 +19,7 @@
 
 #include <QMutex>
 #include <QQuickWindow>
+#include <QReadWriteLock>
 #include <QSGSimpleTextureNode>
 #include <akfrac.h>
 #include <akvideoconverter.h>
@@ -31,7 +32,8 @@ class VideoDisplayPrivate
     public:
         AkVideoConverter m_videoConverter {{AkVideoCaps::Format_argbpack, 0, 0, {}}};
         QImage m_frame;
-        QMutex m_mutex;
+        QMutex m_inputMutex;
+        QReadWriteLock m_updateMutex;
         bool m_fillDisplay {false};
 };
 
@@ -59,16 +61,16 @@ QSGNode *VideoDisplay::updatePaintNode(QSGNode *oldNode,
 {
     Q_UNUSED(updatePaintNodeData)
 
-    this->d->m_mutex.lock();
+    this->d->m_updateMutex.lockForRead();
 
     if (this->d->m_frame.isNull()) {
-        this->d->m_mutex.unlock();
+        this->d->m_updateMutex.unlock();
 
         return nullptr;
     }
 
     auto frame = this->d->m_frame.copy();
-    this->d->m_mutex.unlock();
+    this->d->m_updateMutex.unlock();
 
     if (this->window()->rendererInterface()->graphicsApi() == QSGRendererInterface::Software) {
         Qt::TransformationMode mode = this->smooth()?
@@ -119,26 +121,29 @@ QSGNode *VideoDisplay::updatePaintNode(QSGNode *oldNode,
 
 void VideoDisplay::iStream(const AkPacket &packet)
 {
-    this->d->m_videoConverter.begin();
-    auto src = this->d->m_videoConverter.convert(packet);
-    this->d->m_videoConverter.end();
+    if (this->d->m_inputMutex.tryLock()) {
+        this->d->m_videoConverter.begin();
+        auto src = this->d->m_videoConverter.convert(packet);
+        this->d->m_videoConverter.end();
 
-    this->d->m_mutex.lock();
-    this->d->m_frame = QImage(src.caps().width(),
-                              src.caps().height(),
-                              QImage::Format_ARGB32);
-    auto lineSize =
+        this->d->m_updateMutex.lockForWrite();
+        this->d->m_frame = QImage(src.caps().width(),
+                                  src.caps().height(),
+                                  QImage::Format_ARGB32);
+        auto lineSize =
             qMin<size_t>(src.lineSize(0), this->d->m_frame.bytesPerLine());
 
-    for (int y = 0; y < src.caps().height(); y++) {
-        auto srcLine = src.constLine(0, y);
-        auto dstLine = this->d->m_frame.scanLine(y);
-        memcpy(dstLine, srcLine, lineSize);
+        for (int y = 0; y < src.caps().height(); y++) {
+            auto srcLine = src.constLine(0, y);
+            auto dstLine = this->d->m_frame.scanLine(y);
+            memcpy(dstLine, srcLine, lineSize);
+        }
+
+        this->d->m_updateMutex.unlock();
+
+        QMetaObject::invokeMethod(this, "update");
+        this->d->m_inputMutex.unlock();
     }
-
-    this->d->m_mutex.unlock();
-
-    QMetaObject::invokeMethod(this, "update");
 }
 
 void VideoDisplay::setFillDisplay(bool fillDisplay)
@@ -155,4 +160,10 @@ void VideoDisplay::resetFillDisplay()
     this->setFillDisplay(false);
 }
 
+void VideoDisplay::registerTypes()
+{
+    // @uri Webcamoid
+    qmlRegisterType<VideoDisplay>("Webcamoid", 1, 0, "VideoDisplay");
+}
+
 #include "moc_videodisplay.cpp"
