@@ -77,7 +77,7 @@ struct VideoFormat
     static inline const VideoFormat *formats()
     {
         static const VideoFormat akVideoFormatSpecTable[] {
-            {AkVideoCaps::Format_0bgr,
+            {AkVideoCaps::Format_xbgr,
              VFT_RGB,
              Q_BYTE_ORDER,
              1,
@@ -85,23 +85,7 @@ struct VideoFormat
                    {CT_G, 4, 2, 0, 1, 8, 0, 0},
                    {CT_R, 4, 3, 0, 1, 8, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_0bgrpackbe,
-             VFT_RGB,
-             Q_BIG_ENDIAN,
-             1,
-             {{3, {{CT_B, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_0bgrpackle,
-             VFT_RGB,
-             Q_LITTLE_ENDIAN,
-             1,
-             {{3, {{CT_B, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_0bgr444be,
+            {AkVideoCaps::Format_xbgr444be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
@@ -109,7 +93,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  4, 2, 4, 0, 0},
                    {CT_R, 2, 0,  0, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_0bgr444le,
+            {AkVideoCaps::Format_xbgr444le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
@@ -117,7 +101,23 @@ struct VideoFormat
                    {CT_G, 2, 0,  4, 2, 4, 0, 0},
                    {CT_R, 2, 0,  0, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_0rgb,
+            {AkVideoCaps::Format_xbgr2101010be,
+             VFT_RGB,
+             Q_BIG_ENDIAN,
+             1,
+             {{3, {{CT_B, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  0, 4, 10, 0, 0}}, 32}
+             }},
+            {AkVideoCaps::Format_xbgr2101010le,
+             VFT_RGB,
+             Q_LITTLE_ENDIAN,
+             1,
+             {{3, {{CT_B, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  0, 4, 10, 0, 0}}, 32}
+             }},
+            {AkVideoCaps::Format_xrgb,
              VFT_RGB,
              Q_BYTE_ORDER,
              1,
@@ -125,23 +125,23 @@ struct VideoFormat
                    {CT_G, 4, 2, 0, 1, 8, 0, 0},
                    {CT_B, 4, 3, 0, 1, 8, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_0rgbpackbe,
+            {AkVideoCaps::Format_xrgb2101010be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
-             {{3, {{CT_R, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  0, 4, 8, 0, 0}}, 32}
+             {{3, {{CT_R, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  0, 4, 10, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_0rgbpackle,
+            {AkVideoCaps::Format_xrgb2101010le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
-             {{3, {{CT_R, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  0, 4, 8, 0, 0}}, 32}
+             {{3, {{CT_R, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  0, 4, 10, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_0yuv,
+            {AkVideoCaps::Format_xyuv,
              VFT_YUV,
              Q_BYTE_ORDER,
              1,
@@ -176,6 +176,24 @@ struct VideoFormat
                    {CT_G, 2, 0,  5, 2, 5, 0, 0},
                    {CT_R, 2, 0,  0, 2, 5, 0, 0}}, 16}
              }},
+            {AkVideoCaps::Format_abgr2101010be,
+             VFT_RGB,
+             Q_BIG_ENDIAN,
+             1,
+             {{4, {{CT_A, 4, 0, 30, 4,  2, 0, 0},
+                   {CT_B, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  0, 4, 10, 0, 0}}, 32}
+             }},
+            {AkVideoCaps::Format_abgr2101010le,
+             VFT_RGB,
+             Q_LITTLE_ENDIAN,
+             1,
+             {{4, {{CT_A, 4, 0, 30, 4,  2, 0, 0},
+                   {CT_B, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  0, 4, 10, 0, 0}}, 32}
+             }},
             {AkVideoCaps::Format_abgr4444be,
              VFT_RGB,
              Q_BIG_ENDIAN,
@@ -203,42 +221,6 @@ struct VideoFormat
                    {CT_G, 4, 2, 0, 1, 8, 0, 0},
                    {CT_B, 4, 3, 0, 1, 8, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_argbpackbe,
-             VFT_RGB,
-             Q_BIG_ENDIAN,
-             1,
-             {{4, {{CT_A, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_R, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_argbpackle,
-             VFT_RGB,
-             Q_LITTLE_ENDIAN,
-             1,
-             {{4, {{CT_A, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_R, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_abgrpackbe,
-             VFT_RGB,
-             Q_BIG_ENDIAN,
-             1,
-             {{4, {{CT_A, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_B, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_abgrpackle,
-             VFT_RGB,
-             Q_LITTLE_ENDIAN,
-             1,
-             {{4, {{CT_A, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_B, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_G, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
             {AkVideoCaps::Format_argb1555be,
              VFT_RGB,
              Q_BIG_ENDIAN,
@@ -297,39 +279,39 @@ struct VideoFormat
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
-             {{4, {{CT_A, 8, 0, 30, 2,  2, 0, 0},
-                   {CT_R, 8, 0, 20, 2, 10, 0, 0},
-                   {CT_G, 8, 0, 10, 2, 10, 0, 0},
-                   {CT_B, 8, 0,  0, 2, 10, 0, 0}}, 32}
+             {{4, {{CT_A, 4, 0, 30, 4,  2, 0, 0},
+                   {CT_R, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  0, 4, 10, 0, 0}}, 32}
              }},
             {AkVideoCaps::Format_argb2101010le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
-             {{4, {{CT_A, 8, 0, 30, 2,  2, 0, 0},
-                   {CT_R, 8, 0, 20, 2, 10, 0, 0},
-                   {CT_G, 8, 0, 10, 2, 10, 0, 0},
-                   {CT_B, 8, 0,  0, 2, 10, 0, 0}}, 32}
+             {{4, {{CT_A, 4, 0, 30, 4,  2, 0, 0},
+                   {CT_R, 4, 0, 20, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  0, 4, 10, 0, 0}}, 32}
              }},
             {AkVideoCaps::Format_rgba1010102be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
-             {{4, {{CT_R, 8, 0, 22, 2, 10, 0, 0},
-                   {CT_G, 8, 0, 12, 2, 10, 0, 0},
-                   {CT_B, 8, 0,  2, 2, 10, 0, 0},
-                   {CT_A, 8, 0,  0, 2,  2, 0, 0}}, 32}
+             {{4, {{CT_R, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  2, 4, 10, 0, 0},
+                   {CT_A, 4, 0,  0, 4,  2, 0, 0}}, 32}
              }},
             {AkVideoCaps::Format_rgba1010102le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
-             {{4, {{CT_R, 8, 0, 22, 2, 10, 0, 0},
-                   {CT_G, 8, 0, 12, 2, 10, 0, 0},
-                   {CT_B, 8, 0,  2, 2, 10, 0, 0},
-                   {CT_A, 8, 0,  0, 2,  2, 0, 0}}, 32}
+             {{4, {{CT_R, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  2, 4, 10, 0, 0},
+                   {CT_A, 4, 0,  0, 4,  2, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_rgb0444be,
+            {AkVideoCaps::Format_rgbx444be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
@@ -337,7 +319,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  8, 2, 4, 0, 0},
                    {CT_B, 2, 0,  4, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_rgb0444le,
+            {AkVideoCaps::Format_rgbx444le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
@@ -345,23 +327,12 @@ struct VideoFormat
                    {CT_G, 2, 0,  8, 2, 4, 0, 0},
                    {CT_B, 2, 0,  4, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_ayuvpackbe,
-             VFT_YUV,
-             Q_BIG_ENDIAN,
+            {AkVideoCaps::Format_ay88,
+             VFT_Gray,
+             Q_BYTE_ORDER,
              1,
-             {{4, {{CT_A, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_Y, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_U, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_V, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_ayuvpackle,
-             VFT_YUV,
-             Q_LITTLE_ENDIAN,
-             1,
-             {{4, {{CT_A, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_Y, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_U, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_V, 4, 0,  0, 4, 8, 0, 0}}, 32}
+             {{2, {{CT_A, 2, 0, 0, 1, 8, 0, 0},
+                   {CT_Y, 2, 1, 0, 1, 8, 0, 0}}, 16}
              }},
             {AkVideoCaps::Format_ayuv,
              VFT_YUV,
@@ -390,7 +361,7 @@ struct VideoFormat
                    {CT_U, 8, 4, 0, 2, 16, 0, 0},
                    {CT_V, 8, 6, 0, 2, 16, 0, 0}}, 64}
              }},
-            {AkVideoCaps::Format_bgr0,
+            {AkVideoCaps::Format_bgrx,
              VFT_RGB,
              Q_BYTE_ORDER,
              1,
@@ -406,23 +377,23 @@ struct VideoFormat
                    {CT_G, 3, 1, 0, 1, 8, 0, 0},
                    {CT_R, 3, 2, 0, 1, 8, 0, 0}}, 24}
              }},
-            {AkVideoCaps::Format_bgr0packbe,
+            {AkVideoCaps::Format_bgrx1010102be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
-             {{3, {{CT_B, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  8, 4, 8, 0, 0}}, 32}
+             {{3, {{CT_B, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  2, 4, 10, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_bgr0packle,
+            {AkVideoCaps::Format_bgrx1010102le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
-             {{3, {{CT_B, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  8, 4, 8, 0, 0}}, 32}
+             {{3, {{CT_B, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  2, 4, 10, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_bgr0444be,
+            {AkVideoCaps::Format_bgrx444be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
@@ -430,7 +401,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  8, 2, 4, 0, 0},
                    {CT_R, 2, 0,  4, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_bgr0444le,
+            {AkVideoCaps::Format_bgrx444le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
@@ -438,7 +409,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  8, 2, 4, 0, 0},
                    {CT_R, 2, 0,  4, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_bgr0555be,
+            {AkVideoCaps::Format_bgrx555be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
@@ -446,7 +417,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  6, 2, 4, 0, 0},
                    {CT_R, 2, 0,  1, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_bgr0555le,
+            {AkVideoCaps::Format_bgrx555le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
@@ -518,23 +489,23 @@ struct VideoFormat
                    {CT_G, 2, 0,  5, 2, 6, 0, 0},
                    {CT_R, 2, 0,  0, 2, 5, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_bgrapackbe,
+            {AkVideoCaps::Format_bgra1010102be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
-             {{4, {{CT_B, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_A, 4, 0,  0, 4, 8, 0, 0}}, 32}
+             {{4, {{CT_B, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  2, 4, 10, 0, 0},
+                   {CT_A, 4, 0,  0, 4,  2, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_bgrapackle,
+            {AkVideoCaps::Format_bgra1010102le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
-             {{4, {{CT_B, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_R, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_A, 4, 0,  0, 4, 8, 0, 0}}, 32}
+             {{4, {{CT_B, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_R, 4, 0,  2, 4, 10, 0, 0},
+                   {CT_A, 4, 0,  0, 4,  2, 0, 0}}, 32}
              }},
             {AkVideoCaps::Format_bgr233,
              VFT_RGB,
@@ -774,125 +745,111 @@ struct VideoFormat
               {1, {{CT_B, 2, 0, 0, 2, 9, 0, 0}}, 16},
               {1, {{CT_R, 2, 0, 0, 2, 9, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray9be,
+            {AkVideoCaps::Format_y9be,
              VFT_Gray,
              Q_BIG_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 9, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray9le,
+            {AkVideoCaps::Format_y9le,
              VFT_Gray,
              Q_LITTLE_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 9, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray10be,
+            {AkVideoCaps::Format_y10be,
              VFT_Gray,
              Q_BIG_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 10, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray10le,
+            {AkVideoCaps::Format_y10le,
              VFT_Gray,
              Q_LITTLE_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 10, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray12be,
+            {AkVideoCaps::Format_y12be,
              VFT_Gray,
              Q_BIG_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 12, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray12le,
+            {AkVideoCaps::Format_y12le,
              VFT_Gray,
              Q_LITTLE_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 12, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray14be,
+            {AkVideoCaps::Format_y14be,
              VFT_Gray,
              Q_BIG_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 14, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray14le,
+            {AkVideoCaps::Format_y14le,
              VFT_Gray,
              Q_LITTLE_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 14, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray16be,
+            {AkVideoCaps::Format_y16be,
              VFT_Gray,
              Q_BIG_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 16, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray16le,
+            {AkVideoCaps::Format_y16le,
              VFT_Gray,
              Q_LITTLE_ENDIAN,
              1,
              {{1, {{CT_Y, 2, 0, 0, 2, 16, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_gray32be,
+            {AkVideoCaps::Format_y32be,
              VFT_Gray,
              Q_BIG_ENDIAN,
              1,
              {{1, {{CT_Y, 4, 0, 0, 4, 32, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_gray32le,
+            {AkVideoCaps::Format_y32le,
              VFT_Gray,
              Q_LITTLE_ENDIAN,
              1,
              {{1, {{CT_Y, 4, 0, 0, 4, 32, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_gray4,
+            {AkVideoCaps::Format_xy44,
              VFT_Gray,
              Q_BYTE_ORDER,
              1,
              {{1, {{CT_Y, 1, 0, 0, 1, 4, 0, 0}}, 8}
              }},
-            {AkVideoCaps::Format_gray6,
+            {AkVideoCaps::Format_xy26,
              VFT_Gray,
              Q_BYTE_ORDER,
              1,
              {{1, {{CT_Y, 1, 0, 0, 1, 6, 0, 0}}, 8}
              }},
-            {AkVideoCaps::Format_gray8,
+            {AkVideoCaps::Format_y8,
              VFT_Gray,
              Q_BYTE_ORDER,
              1,
              {{1, {{CT_Y, 1, 0, 0, 1, 8, 0, 0}}, 8}
              }},
-            {AkVideoCaps::Format_graya8,
+            {AkVideoCaps::Format_ya88,
              VFT_Gray,
              Q_BYTE_ORDER,
              1,
              {{2, {{CT_Y, 2, 0, 0, 1, 8, 0, 0},
                    {CT_A, 2, 1, 0, 1, 8, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_graya8packbe,
-             VFT_Gray,
-             Q_BIG_ENDIAN,
-             1,
-             {{2, {{CT_Y, 2, 0, 8, 2, 8, 0, 0},
-                   {CT_A, 2, 0, 0, 2, 8, 0, 0}}, 16}
-             }},
-            {AkVideoCaps::Format_graya8packle,
-             VFT_Gray,
-             Q_LITTLE_ENDIAN,
-             1,
-             {{2, {{CT_Y, 2, 0, 8, 2, 8, 0, 0},
-                   {CT_A, 2, 0, 0, 2, 8, 0, 0}}, 16}
-             }},
-            {AkVideoCaps::Format_graya16be,
+            {AkVideoCaps::Format_ya16be,
              VFT_Gray,
              Q_BIG_ENDIAN,
              1,
              {{2, {{CT_Y, 4, 0, 0, 2, 16, 0, 0},
                    {CT_A, 4, 2, 0, 2, 16, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_graya16le,
+            {AkVideoCaps::Format_ya16le,
              VFT_Gray,
              Q_LITTLE_ENDIAN,
              1,
@@ -1068,7 +1025,7 @@ struct VideoFormat
               {2, {{CT_U, 4, 0, 0, 2, 16, 0, 0},
                    {CT_V, 4, 2, 0, 2, 16, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_rgb0,
+            {AkVideoCaps::Format_rgbx,
              VFT_RGB,
              Q_BYTE_ORDER,
              1,
@@ -1140,7 +1097,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  5, 2, 5, 0, 0},
                    {CT_B, 2, 0,  0, 2, 5, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_rgb5550be,
+            {AkVideoCaps::Format_rgbx5551be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
@@ -1148,7 +1105,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  6, 2, 5, 0, 0},
                    {CT_B, 2, 0,  1, 2, 5, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_rgb5550le,
+            {AkVideoCaps::Format_rgbx5551le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
@@ -1197,39 +1154,21 @@ struct VideoFormat
                    {CT_B, 4, 2, 0, 1, 8, 0, 0},
                    {CT_A, 4, 3, 0, 1, 8, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_rgbapackbe,
+            {AkVideoCaps::Format_rgbx1010102be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
-             {{4, {{CT_R, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_A, 4, 0,  0, 4, 8, 0, 0}}, 32}
+             {{3, {{CT_R, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  2, 4, 10, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_rgbapackle,
+            {AkVideoCaps::Format_rgbx1010102le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
-             {{4, {{CT_R, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  8, 4, 8, 0, 0},
-                   {CT_A, 4, 0,  0, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_rgb0packbe,
-             VFT_RGB,
-             Q_BIG_ENDIAN,
-             1,
-             {{3, {{CT_R, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  8, 4, 8, 0, 0}}, 32}
-             }},
-            {AkVideoCaps::Format_rgb0packle,
-             VFT_RGB,
-             Q_LITTLE_ENDIAN,
-             1,
-             {{3, {{CT_R, 4, 0, 24, 4, 8, 0, 0},
-                   {CT_G, 4, 0, 16, 4, 8, 0, 0},
-                   {CT_B, 4, 0,  8, 4, 8, 0, 0}}, 32}
+             {{3, {{CT_R, 4, 0, 22, 4, 10, 0, 0},
+                   {CT_G, 4, 0, 12, 4, 10, 0, 0},
+                   {CT_B, 4, 0,  2, 4, 10, 0, 0}}, 32}
              }},
             {AkVideoCaps::Format_rgba4444be,
              VFT_RGB,
@@ -1249,7 +1188,7 @@ struct VideoFormat
                    {CT_B, 2, 0,  4, 2, 4, 0, 0},
                    {CT_A, 2, 0,  0, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_rgb0555be,
+            {AkVideoCaps::Format_rgbx555be,
              VFT_RGB,
              Q_BIG_ENDIAN,
              1,
@@ -1257,7 +1196,7 @@ struct VideoFormat
                    {CT_G, 2, 0,  6, 2, 5, 0, 0},
                    {CT_B, 2, 0,  1, 2, 5, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_rgb0555le,
+            {AkVideoCaps::Format_rgbx555le,
              VFT_RGB,
              Q_LITTLE_ENDIAN,
              1,
@@ -1335,6 +1274,46 @@ struct VideoFormat
                    {CT_V, 4, 2, 0, 1, 8, 1, 0}}, 16},
               {1, {{CT_A, 1, 0, 0, 1, 8, 0, 0}},  8}
              }},
+            {AkVideoCaps::Format_v210be,
+             VFT_YUV,
+             Q_BIG_ENDIAN,
+             1,
+             {{3, {{CT_U, 4, 0, 20, 4, 10, 1, 0},
+                   {CT_Y, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_V, 4, 0,  0, 4, 10, 1, 0}}, 22}
+             }},
+            {AkVideoCaps::Format_v210le,
+             VFT_YUV,
+             Q_LITTLE_ENDIAN,
+             1,
+             {{3, {{CT_U, 4, 0, 20, 4, 10, 1, 0},
+                   {CT_Y, 4, 0, 10, 4, 10, 0, 0},
+                   {CT_V, 4, 0,  0, 4, 10, 1, 0}}, 22}
+             }},
+            {AkVideoCaps::Format_v216be,
+             VFT_YUV,
+             Q_BIG_ENDIAN,
+             1,
+             {{3, {{CT_U, 8, 0, 0, 2, 16, 1, 0},
+                   {CT_Y, 4, 2, 0, 2, 16, 0, 0},
+                   {CT_V, 8, 4, 0, 2, 16, 1, 0}}, 32}
+             }},
+            {AkVideoCaps::Format_v216le,
+             VFT_YUV,
+             Q_LITTLE_ENDIAN,
+             1,
+             {{3, {{CT_U, 8, 0, 0, 2, 16, 1, 0},
+                   {CT_Y, 4, 2, 0, 2, 16, 0, 0},
+                   {CT_V, 8, 4, 0, 2, 16, 1, 0}}, 32}
+             }},
+            {AkVideoCaps::Format_v308,
+             VFT_YUV,
+             Q_BYTE_ORDER,
+             1,
+             {{3, {{CT_U, 1, 0, 0, 1, 8, 0, 0},
+                   {CT_Y, 1, 1, 0, 1, 8, 0, 0},
+                   {CT_V, 1, 2, 0, 1, 8, 0, 0}}, 24}
+             }},
             {AkVideoCaps::Format_uyva,
              VFT_YUV,
              Q_BYTE_ORDER,
@@ -1369,7 +1348,7 @@ struct VideoFormat
                    {CT_Y, 4, 2, 0, 1, 8, 0, 0},
                    {CT_A, 4, 3, 0, 1, 8, 0, 0}}, 32}
              }},
-            {AkVideoCaps::Format_vuy0,
+            {AkVideoCaps::Format_vuyx,
              VFT_YUV,
              Q_BYTE_ORDER,
              1,
@@ -1769,7 +1748,7 @@ struct VideoFormat
               {1, {{CT_U, 2, 0, 0, 2, 9, 0, 0}}, 16},
               {1, {{CT_V, 2, 0, 0, 2, 9, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_yuv444packbe,
+            {AkVideoCaps::Format_xyuv4444be,
              VFT_YUV,
              Q_BIG_ENDIAN,
              1,
@@ -1777,7 +1756,7 @@ struct VideoFormat
                    {CT_U, 2, 0, 4, 2, 4, 0, 0},
                    {CT_V, 2, 0, 0, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_yuv444packle,
+            {AkVideoCaps::Format_xyuv4444le,
              VFT_YUV,
              Q_LITTLE_ENDIAN,
              1,
@@ -1785,7 +1764,7 @@ struct VideoFormat
                    {CT_U, 2, 0, 4, 2, 4, 0, 0},
                    {CT_V, 2, 0, 0, 2, 4, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_yuv555packbe,
+            {AkVideoCaps::Format_xyuv1555be,
              VFT_YUV,
              Q_BIG_ENDIAN,
              1,
@@ -1793,7 +1772,7 @@ struct VideoFormat
                    {CT_U, 2, 0,  5, 2, 5, 0, 0},
                    {CT_V, 2, 0,  0, 2, 5, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_yuv555packle,
+            {AkVideoCaps::Format_xyuv1555le,
              VFT_YUV,
              Q_LITTLE_ENDIAN,
              1,
@@ -1801,7 +1780,7 @@ struct VideoFormat
                    {CT_U, 2, 0,  5, 2, 5, 0, 0},
                    {CT_V, 2, 0,  0, 2, 5, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_yuv565packbe,
+            {AkVideoCaps::Format_yuv565be,
              VFT_YUV,
              Q_BIG_ENDIAN,
              1,
@@ -1809,7 +1788,7 @@ struct VideoFormat
                    {CT_U, 2, 0,  5, 2, 6, 0, 0},
                    {CT_V, 2, 0,  0, 2, 5, 0, 0}}, 16}
              }},
-            {AkVideoCaps::Format_yuv565packle,
+            {AkVideoCaps::Format_yuv565le,
              VFT_YUV,
              Q_LITTLE_ENDIAN,
              1,
@@ -2534,10 +2513,8 @@ void AkVideoCaps::registerTypes()
 {
     qRegisterMetaType<AkVideoCaps>("AkVideoCaps");
     qRegisterMetaType<AkVideoCapsList>("AkVideoCapsList");
-    qRegisterMetaTypeStreamOperators<AkVideoCaps>("AkVideoCaps");
     qRegisterMetaType<PixelFormat>("PixelFormat");
     qRegisterMetaType<PixelFormatList>("PixelFormatList");
-    QMetaType::registerDebugStreamOperator<AkVideoCaps::PixelFormat>();
     qmlRegisterSingletonType<AkVideoCaps>("Ak", 1, 0, "AkVideoCaps",
                                           [] (QQmlEngine *qmlEngine,
                                               QJSEngine *jsEngine) -> QObject * {
