@@ -1,5 +1,5 @@
 /* Webcamoid, webcam capture application.
- * Copyright (C) 2016  Gonzalo Exequiel Pedone
+ * Copyright (C) 2023  Gonzalo Exequiel Pedone
  *
  * Webcamoid is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,21 +18,19 @@
  */
 
 #include "plugin.h"
-#include "audiodevqtaudio.h"
+#include "audiodevqt.h"
 
 QObject *Plugin::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_SUBMODULE)
-        return new AudioDevQtAudio();
-
-    return nullptr;
+    return new AudioDevQt();
 }
 
 QStringList Plugin::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_plugin.cpp"
