@@ -22,17 +22,15 @@
 
 QObject *Cartoon::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new CartoonElement();
-
-    return nullptr;
+    return new CartoonElement();
 }
 
 QStringList Cartoon::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_cartoon.cpp"
