@@ -22,17 +22,15 @@
 
 QObject *Convolve::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new ConvolveElement();
-
-    return nullptr;
+    return new ConvolveElement();
 }
 
 QStringList Convolve::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_convolve.cpp"
