@@ -22,17 +22,15 @@
 
 QObject *Edge::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new EdgeElement();
-
-    return nullptr;
+    return new EdgeElement();
 }
 
 QStringList Edge::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_edge.cpp"
