@@ -17,20 +17,33 @@
  * Web-Site: http://webcamoid.github.io/
  */
 
-#include <QImage>
 #include <QQmlContext>
+#include <QSize>
+#include <qrgb.h>
+#include <akfrac.h>
 #include <akpacket.h>
+#include <akvideocaps.h>
+#include <akvideoconverter.h>
 #include <akvideopacket.h>
 
 #include "frameoverlapelement.h"
 
+struct SumPixel
+{
+    quint64 r;
+    quint64 g;
+    quint64 b;
+    quint64 a;
+};
+
 class FrameOverlapElementPrivate
 {
     public:
         int m_nFrames {16};
         int m_stride {4};
-        QVector<QImage> m_frames;
+        QVector<AkVideoPacket> m_frames;
         QSize m_frameSize;
+        AkVideoConverter m_videoConverter {{AkVideoCaps::Format_argbpack, 0, 0, {}}};
 };
 
 FrameOverlapElement::FrameOverlapElement(): AkElement()
@@ -71,64 +84,93 @@ void FrameOverlapElement::controlInterfaceConfigure(QQmlContext *context,
 
 AkPacket FrameOverlapElement::iVideoStream(const AkVideoPacket &packet)
 {
-    auto src = packet.toImage();
+    this->d->m_videoConverter.begin();
+    auto src = this->d->m_videoConverter.convert(packet);
+    this->d->m_videoConverter.end();
 
-    if (src.isNull())
-        return AkPacket();
+    if (!src)
+        return {};
 
-    src = src.convertToFormat(QImage::Format_ARGB32);
-    QImage oFrame(src.size(), src.format());
+    QSize frameSize(src.caps().width(), src.caps().height());
 
-    if (src.size() != this->d->m_frameSize) {
+    if (frameSize != this->d->m_frameSize) {
         this->d->m_frames.clear();
-        this->d->m_frameSize = src.size();
+        this->d->m_frameSize = frameSize;
     }
 
-    this->d->m_frames << src.copy();
+    this->d->m_frames << src;
     int diff = this->d->m_frames.size() - this->d->m_nFrames;
 
     for (int i = 0; i < diff; i++)
         this->d->m_frames.removeFirst();
 
-    int stride = this->d->m_stride > 0? this->d->m_stride: 1;
+    int stride = qMax(this->d->m_stride, 1);
 
-    for (int y = 0; y < oFrame.height(); y++) {
-        QRgb *dstBits = reinterpret_cast<QRgb *>(oFrame.scanLine(y));
+    AkVideoPacket dst(src.caps());
+    dst.copyMetadata(src);
 
-        for (int x = 0; x < oFrame.width(); x++) {
-            int r = 0;
-            int g = 0;
-            int b = 0;
-            int a = 0;
-            int n = 0;
+    auto sumFrameSize = size_t(dst.caps().width())
+                        * size_t(dst.caps().height());
+    auto sumFrame = new SumPixel[sumFrameSize];
+    memset(sumFrame, 0, sumFrameSize * sizeof(SumPixel));
+    int nFrames = 0;
 
-            for (int frame = this->d->m_frames.size() - 1;
-                 frame >= 0;
-                 frame -= stride) {
-                QRgb pixel = this->d->m_frames[frame].pixel(x, y);
+    for (int i = this->d->m_frames.size() - 1;
+         i >= 0;
+         i -= stride) {
+        auto &frame = this->d->m_frames[i];
+        auto dstLine = sumFrame;
 
-                r += qRed(pixel);
-                g += qGreen(pixel);
-                b += qBlue(pixel);
-                a += qAlpha(pixel);
-                n++;
+        for (int y = 0; y < dst.caps().height(); y++) {
+            auto srcLine = reinterpret_cast<const QRgb *>(frame.constLine(0, y));
+
+            for (int x = 0; x < dst.caps().width(); x++) {
+                auto &ipixel = srcLine[x];
+                auto &opixel = dstLine[x];
+
+                opixel.r += qRed(ipixel);
+                opixel.g += qGreen(ipixel);
+                opixel.b += qBlue(ipixel);
+                opixel.a += qAlpha(ipixel);
             }
 
-            if (n > 0) {
-                r /= n;
-                g /= n;
-                b /= n;
-                a /= n;
-
-                dstBits[x] = qRgba(r, g, b, a);
-            } else {
-                dstBits[x] = qRgba(0, 0, 0, 0);
-            }
+            dstLine += dst.caps().width();
         }
+
+        nFrames++;
     }
 
-    auto oPacket = AkVideoPacket::fromImage(oFrame, packet);
-    akSend(oPacket)
+    if (nFrames < 1) {
+        delete [] sumFrame;
+
+        if (src)
+            emit this->oStream(src);
+
+        return src;
+    }
+
+    auto srcLine = sumFrame;
+
+    for (int y = 0; y < dst.caps().height(); y++) {
+        auto dstBits = reinterpret_cast<QRgb *>(dst.line(0, y));
+
+        for (int x = 0; x < dst.caps().width(); x++) {
+            auto &ipixel = srcLine[x];
+            dstBits[x] = qRgba(ipixel.r / nFrames,
+                               ipixel.g / nFrames,
+                               ipixel.b / nFrames,
+                               ipixel.a / nFrames);
+        }
+
+        srcLine += dst.caps().width();
+    }
+
+    delete [] sumFrame;
+
+    if (dst)
+        emit this->oStream(dst);
+
+    return dst;
 }
 
 void FrameOverlapElement::setNFrames(int nFrames)
