@@ -22,17 +22,15 @@
 
 QObject *Implode::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new ImplodeElement();
-
-    return nullptr;
+    return new ImplodeElement();
 }
 
 QStringList Implode::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_implode.cpp"
