@@ -22,17 +22,15 @@
 
 QObject *Invert::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new InvertElement();
-
-    return nullptr;
+    return new InvertElement();
 }
 
 QStringList Invert::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_invert.cpp"
