@@ -22,17 +22,15 @@
 
 QObject *Life::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new LifeElement();
-
-    return nullptr;
+    return new LifeElement();
 }
 
 QStringList Life::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_life.cpp"
