@@ -22,17 +22,15 @@
 
 QObject *MatrixTransform::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new MatrixTransformElement();
-
-    return nullptr;
+    return new MatrixTransformElement();
 }
 
 QStringList MatrixTransform::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_matrixtransform.cpp"
