@@ -21,7 +21,6 @@
 #include <QMutex>
 #include <QSharedPointer>
 #include <QWaitCondition>
-#include <akaudiocaps.h>
 #include <akaudioconverter.h>
 #include <akaudiopacket.h>
 #include <akcaps.h>
@@ -37,8 +36,123 @@ extern "C"
 #include "audiostream.h"
 #include "mediawriterffmpeg.h"
 
-using SampleFormatsMap = QMap<AkAudioCaps::SampleFormat, AVSampleFormat>;
-using ChannelLayoutsMap = QMap<AkAudioCaps::ChannelLayout, uint64_t>;
+struct SampleFormat
+{
+    AkAudioCaps::SampleFormat akFormat;
+    AVSampleFormat ffFormat;
+
+    static inline const SampleFormat *byAk(bool planar, AkAudioCaps::SampleFormat akFormat);
+    static inline const SampleFormat *byFF(bool planar, AVSampleFormat ffFormat);
+};
+
+static const SampleFormat multiSinkSampleFormatTable[] {
+    {AkAudioCaps::SampleFormat_u8  , AV_SAMPLE_FMT_U8  },
+    {AkAudioCaps::SampleFormat_s16 , AV_SAMPLE_FMT_S16 },
+    {AkAudioCaps::SampleFormat_s32 , AV_SAMPLE_FMT_S32 },
+    {AkAudioCaps::SampleFormat_s64 , AV_SAMPLE_FMT_S64 },
+    {AkAudioCaps::SampleFormat_flt , AV_SAMPLE_FMT_FLT },
+    {AkAudioCaps::SampleFormat_dbl , AV_SAMPLE_FMT_DBL },
+    {AkAudioCaps::SampleFormat_none, AV_SAMPLE_FMT_NONE},
+};
+
+static const SampleFormat multiSinkPlanarSampleFormatTable[] {
+    {AkAudioCaps::SampleFormat_u8  , AV_SAMPLE_FMT_U8P },
+    {AkAudioCaps::SampleFormat_s16 , AV_SAMPLE_FMT_S16P},
+    {AkAudioCaps::SampleFormat_s32 , AV_SAMPLE_FMT_S32P},
+    {AkAudioCaps::SampleFormat_s64 , AV_SAMPLE_FMT_S64P},
+    {AkAudioCaps::SampleFormat_flt , AV_SAMPLE_FMT_FLTP},
+    {AkAudioCaps::SampleFormat_dbl , AV_SAMPLE_FMT_DBLP},
+    {AkAudioCaps::SampleFormat_none, AV_SAMPLE_FMT_NONE},
+};
+
+const SampleFormat *SampleFormat::byFF(bool planar, AVSampleFormat ffFormat)
+{
+    auto fmt = planar?
+                   multiSinkPlanarSampleFormatTable:
+                   multiSinkSampleFormatTable;
+
+    for (; fmt->akFormat != AkAudioCaps::SampleFormat_none; fmt++)
+        if (fmt->ffFormat == ffFormat)
+            return fmt;
+
+    return fmt;
+}
+
+const SampleFormat *SampleFormat::byAk(bool planar, AkAudioCaps::SampleFormat akFormat)
+{
+    auto fmt = planar?
+                   multiSinkPlanarSampleFormatTable:
+                   multiSinkSampleFormatTable;
+
+    for (; fmt->akFormat != AkAudioCaps::SampleFormat_none; fmt++)
+        if (fmt->akFormat == akFormat)
+            return fmt;
+
+    return fmt;
+}
+
+struct ChannelLayout
+{
+    uint64_t ffLayout;
+    AkAudioCaps::ChannelLayout akLayout;
+
+    static inline const ChannelLayout *byFF(uint64_t ffLayout);
+    static inline const ChannelLayout *byAk(AkAudioCaps::ChannelLayout akLayout);
+};
+
+static const ChannelLayout multiSrcChannelLayoutsTable[] {
+    {AV_CH_LAYOUT_MONO             , AkAudioCaps::Layout_mono         },
+    {AV_CH_LAYOUT_STEREO           , AkAudioCaps::Layout_stereo       },
+    {AV_CH_LAYOUT_2POINT1          , AkAudioCaps::Layout_2p1          },
+    {AV_CH_LAYOUT_SURROUND         , AkAudioCaps::Layout_3p0          },
+    {AV_CH_LAYOUT_2_1              , AkAudioCaps::Layout_3p0_back     },
+    {AV_CH_LAYOUT_3POINT1          , AkAudioCaps::Layout_3p1          },
+    {AV_CH_LAYOUT_4POINT0          , AkAudioCaps::Layout_4p0          },
+    {AV_CH_LAYOUT_QUAD             , AkAudioCaps::Layout_quad         },
+    {AV_CH_LAYOUT_2_2              , AkAudioCaps::Layout_quad_side    },
+    {AV_CH_LAYOUT_4POINT1          , AkAudioCaps::Layout_4p1          },
+    {AV_CH_LAYOUT_5POINT0_BACK     , AkAudioCaps::Layout_5p0          },
+    {AV_CH_LAYOUT_5POINT0          , AkAudioCaps::Layout_5p0_side     },
+    {AV_CH_LAYOUT_5POINT1_BACK     , AkAudioCaps::Layout_5p1          },
+    {AV_CH_LAYOUT_5POINT1          , AkAudioCaps::Layout_5p1_side     },
+    {AV_CH_LAYOUT_6POINT0          , AkAudioCaps::Layout_6p0          },
+    {AV_CH_LAYOUT_6POINT0_FRONT    , AkAudioCaps::Layout_6p0_front    },
+    {AV_CH_LAYOUT_HEXAGONAL        , AkAudioCaps::Layout_hexagonal    },
+    {AV_CH_LAYOUT_6POINT1          , AkAudioCaps::Layout_6p1          },
+    {AV_CH_LAYOUT_6POINT1_BACK     , AkAudioCaps::Layout_6p1_back     },
+    {AV_CH_LAYOUT_6POINT1_FRONT    , AkAudioCaps::Layout_6p1_front    },
+    {AV_CH_LAYOUT_7POINT0          , AkAudioCaps::Layout_7p0          },
+    {AV_CH_LAYOUT_7POINT0_FRONT    , AkAudioCaps::Layout_7p0_front    },
+    {AV_CH_LAYOUT_7POINT1          , AkAudioCaps::Layout_7p1          },
+    {AV_CH_LAYOUT_7POINT1_WIDE     , AkAudioCaps::Layout_7p1_wide     },
+    {AV_CH_LAYOUT_7POINT1_WIDE_BACK, AkAudioCaps::Layout_7p1_wide_back},
+    {AV_CH_LAYOUT_OCTAGONAL        , AkAudioCaps::Layout_octagonal    },
+    {AV_CH_LAYOUT_HEXADECAGONAL    , AkAudioCaps::Layout_hexadecagonal},
+    {AV_CH_LAYOUT_STEREO_DOWNMIX   , AkAudioCaps::Layout_downmix      },
+    {0                             , AkAudioCaps::Layout_none         },
+};
+
+const ChannelLayout *ChannelLayout::byFF(uint64_t ffLayout)
+{
+    auto lyt = multiSrcChannelLayoutsTable;
+
+    for (; lyt->akLayout != AkAudioCaps::Layout_none; lyt++)
+        if (lyt->ffLayout == ffLayout)
+            return lyt;
+
+    return lyt;
+}
+
+const ChannelLayout *ChannelLayout::byAk(AkAudioCaps::ChannelLayout akLayout)
+{
+    auto lyt = multiSrcChannelLayoutsTable;
+
+    for (; lyt->akLayout != AkAudioCaps::Layout_none; lyt++)
+        if (lyt->akLayout == akLayout)
+            return lyt;
+
+    return lyt;
+}
 
 class AudioStreamPrivate
 {
@@ -48,78 +162,6 @@ class AudioStreamPrivate
         QMutex m_frameMutex;
         int64_t m_pts {0};
         QWaitCondition m_frameReady;
-
-        inline static const SampleFormatsMap &sampleFormats(bool planar)
-        {
-            static const SampleFormatsMap formats {
-                {AkAudioCaps::SampleFormat_u8 , AV_SAMPLE_FMT_U8 },
-                {AkAudioCaps::SampleFormat_s16, AV_SAMPLE_FMT_S16},
-                {AkAudioCaps::SampleFormat_s32, AV_SAMPLE_FMT_S32},
-                {AkAudioCaps::SampleFormat_s64, AV_SAMPLE_FMT_S64 },
-                {AkAudioCaps::SampleFormat_flt, AV_SAMPLE_FMT_FLT},
-                {AkAudioCaps::SampleFormat_dbl, AV_SAMPLE_FMT_DBL},
-            };
-            static const SampleFormatsMap planarFormats {
-                {AkAudioCaps::SampleFormat_u8 , AV_SAMPLE_FMT_U8P },
-                {AkAudioCaps::SampleFormat_s16, AV_SAMPLE_FMT_S16P},
-                {AkAudioCaps::SampleFormat_s32, AV_SAMPLE_FMT_S32P},
-                {AkAudioCaps::SampleFormat_s64, AV_SAMPLE_FMT_S64P},
-                {AkAudioCaps::SampleFormat_flt, AV_SAMPLE_FMT_FLTP},
-                {AkAudioCaps::SampleFormat_dbl, AV_SAMPLE_FMT_DBLP},
-            };
-
-            return planar? planarFormats: formats;
-        }
-
-        inline static const QVector<AVSampleFormat> &planarFormats()
-        {
-            static const QVector<AVSampleFormat> formats {
-                AV_SAMPLE_FMT_U8P ,
-                AV_SAMPLE_FMT_S16P,
-                AV_SAMPLE_FMT_S32P,
-                AV_SAMPLE_FMT_FLTP,
-                AV_SAMPLE_FMT_DBLP,
-                AV_SAMPLE_FMT_S64P,
-            };
-
-            return formats;
-        }
-
-        inline static const ChannelLayoutsMap &channelLayouts()
-        {
-            static const ChannelLayoutsMap channelLayouts = {
-                {AkAudioCaps::Layout_mono         , AV_CH_LAYOUT_MONO             },
-                {AkAudioCaps::Layout_stereo       , AV_CH_LAYOUT_STEREO           },
-                {AkAudioCaps::Layout_2p1          , AV_CH_LAYOUT_2POINT1          },
-                {AkAudioCaps::Layout_3p0          , AV_CH_LAYOUT_SURROUND         },
-                {AkAudioCaps::Layout_3p0_back     , AV_CH_LAYOUT_2_1              },
-                {AkAudioCaps::Layout_3p1          , AV_CH_LAYOUT_3POINT1          },
-                {AkAudioCaps::Layout_4p0          , AV_CH_LAYOUT_4POINT0          },
-                {AkAudioCaps::Layout_quad         , AV_CH_LAYOUT_QUAD             },
-                {AkAudioCaps::Layout_quad_side    , AV_CH_LAYOUT_2_2              },
-                {AkAudioCaps::Layout_4p1          , AV_CH_LAYOUT_4POINT1          },
-                {AkAudioCaps::Layout_5p0          , AV_CH_LAYOUT_5POINT0_BACK     },
-                {AkAudioCaps::Layout_5p0_side     , AV_CH_LAYOUT_5POINT0          },
-                {AkAudioCaps::Layout_5p1          , AV_CH_LAYOUT_5POINT1_BACK     },
-                {AkAudioCaps::Layout_5p1_side     , AV_CH_LAYOUT_5POINT1          },
-                {AkAudioCaps::Layout_6p0          , AV_CH_LAYOUT_6POINT0          },
-                {AkAudioCaps::Layout_6p0_front    , AV_CH_LAYOUT_6POINT0_FRONT    },
-                {AkAudioCaps::Layout_hexagonal    , AV_CH_LAYOUT_HEXAGONAL        },
-                {AkAudioCaps::Layout_6p1          , AV_CH_LAYOUT_6POINT1          },
-                {AkAudioCaps::Layout_6p1_back     , AV_CH_LAYOUT_6POINT1_BACK     },
-                {AkAudioCaps::Layout_6p1_front    , AV_CH_LAYOUT_6POINT1_FRONT    },
-                {AkAudioCaps::Layout_7p0          , AV_CH_LAYOUT_7POINT0          },
-                {AkAudioCaps::Layout_7p0_front    , AV_CH_LAYOUT_7POINT0_FRONT    },
-                {AkAudioCaps::Layout_7p1          , AV_CH_LAYOUT_7POINT1          },
-                {AkAudioCaps::Layout_7p1_wide     , AV_CH_LAYOUT_7POINT1_WIDE     },
-                {AkAudioCaps::Layout_7p1_wide_back, AV_CH_LAYOUT_7POINT1_WIDE_BACK},
-                {AkAudioCaps::Layout_octagonal    , AV_CH_LAYOUT_OCTAGONAL        },
-                {AkAudioCaps::Layout_hexadecagonal, AV_CH_LAYOUT_HEXADECAGONAL    },
-                {AkAudioCaps::Layout_downmix      , AV_CH_LAYOUT_STEREO_DOWNMIX   },
-            };
-
-            return channelLayouts;
-        }
 };
 
 AudioStream::AudioStream(const AVFormatContext *formatContext,
@@ -157,23 +199,27 @@ AudioStream::AudioStream(const AVFormatContext *formatContext,
     }
 
     AkAudioCaps audioCaps(configs["caps"].value<AkCaps>());
-    auto ffFormat =
-            AudioStreamPrivate::sampleFormats(audioCaps.planar())
-            .value(audioCaps.format(), AV_SAMPLE_FMT_NONE);
-    auto sampleFormat = QString(av_get_sample_fmt_name(ffFormat));
-    auto supportedSampleFormats =
-            defaultCodecParams["supportedSampleFormats"].toStringList();
 
-    if (!supportedSampleFormats.isEmpty()
-        && !supportedSampleFormats.contains(sampleFormat)) {
-        auto defaultSampleFormat = defaultCodecParams["defaultSampleFormat"].toString();
-        ffFormat = av_get_sample_fmt(defaultSampleFormat.toStdString().c_str());
-        auto planar = AudioStreamPrivate::planarFormats().contains(ffFormat);
-        auto format =
-                AudioStreamPrivate::sampleFormats(planar)
-                .key(ffFormat, AkAudioCaps::SampleFormat_none);
-        audioCaps.setFormat(format);
-        audioCaps.updatePlaneSize(planar);
+    auto supportedSampleFormats =
+            defaultCodecParams["supportedSampleFormats"].toList();
+
+    if (!supportedSampleFormats.isEmpty()) {
+        using FormatPair = QPair<AkAudioCaps::SampleFormat, bool>;
+        FormatPair fmt(audioCaps.format(), audioCaps.planar());
+
+        auto count =
+            std::count_if(supportedSampleFormats.begin(),
+                          supportedSampleFormats.end(),
+                          [&fmt] (const QVariant &var) {
+                              return var.value<FormatPair>() == fmt;
+                          });
+
+        if (count < 1) {
+            auto defaultSampleFormat =
+                defaultCodecParams["defaultSampleFormat"].value<FormatPair>();
+            audioCaps.setFormat(defaultSampleFormat.first);
+            audioCaps.setPlanar(defaultSampleFormat.second);
+        }
     }
 
     auto supportedSampleRates =
@@ -196,35 +242,44 @@ AudioStream::AudioStream(const AVFormatContext *formatContext,
             }
         }
 
-        audioCaps.rate() = sampleRate;
+        audioCaps.setRate(sampleRate);
     }
 
-    auto channelLayout = AkAudioCaps::channelLayoutToString(audioCaps.layout());
+    auto channelLayout = audioCaps.layout();
     auto supportedChannelLayouts =
-            defaultCodecParams["supportedChannelLayouts"].toStringList();
+            defaultCodecParams["supportedChannelLayouts"].toList();
 
-    if (!supportedChannelLayouts.isEmpty() && !supportedChannelLayouts.contains(channelLayout)) {
-        auto defaultChannelLayout = defaultCodecParams["defaultChannelLayout"].toString();
-        auto layout = AkAudioCaps::channelLayoutFromString(defaultChannelLayout);
-        audioCaps.setLayout(layout);
+    if (!supportedChannelLayouts.isEmpty()
+        && !supportedChannelLayouts.contains(channelLayout)) {
+        auto defaultChannelLayout =
+                AkAudioCaps::ChannelLayout(defaultCodecParams["defaultChannelLayout"].toInt());
+        audioCaps.setLayout(defaultChannelLayout);
     }
 
     if (!strcmp(formatContext->oformat->name, "gxf")) {
-        audioCaps.rate() = 48000;
+        audioCaps.setRate(48000);
         audioCaps.setLayout(AkAudioCaps::Layout_mono);
     } else if (!strcmp(formatContext->oformat->name, "mxf")) {
-        audioCaps.rate() = 48000;
+        audioCaps.setRate(48000);
     } else if (!strcmp(formatContext->oformat->name, "swf")) {
         audioCaps = mediaWriter->nearestSWFCaps(audioCaps);
     }
 
     codecContext->sample_fmt =
-            AudioStreamPrivate::sampleFormats(audioCaps.planar())
-            .value(audioCaps.format(), AV_SAMPLE_FMT_NONE);
+        SampleFormat::byAk(audioCaps.planar(), audioCaps.format())->ffFormat;
     codecContext->sample_rate = audioCaps.rate();
-    auto layout = AkAudioCaps::channelLayoutToString(audioCaps.layout());
-    codecContext->channel_layout = av_get_channel_layout(layout.toStdString().c_str());
+    auto layout = ChannelLayout::byAk(audioCaps.layout())->ffLayout;
+
+    if (layout == 0)
+        layout = AV_CH_LAYOUT_STEREO;
+
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+    memset(&codecContext->ch_layout, 0, sizeof(AVChannelLayout));
+    av_channel_layout_from_mask(&codecContext->ch_layout, layout);
+#else
+    codecContext->channel_layout = layout;
     codecContext->channels = audioCaps.channels();
+#endif
 
     auto stream = this->stream();
     stream->time_base.num = 1;
@@ -239,6 +294,17 @@ AudioStream::~AudioStream()
     delete this->d;
 }
 
+AkAudioCaps::SampleFormat AudioStream::sampleFormat(AVSampleFormat format)
+{
+    return SampleFormat::byFF(av_sample_fmt_is_planar(format),
+                              format)->akFormat;
+}
+
+AkAudioCaps::ChannelLayout AudioStream::channelLayout(uint64_t layout)
+{
+    return ChannelLayout::byFF(layout)->akLayout;
+}
+
 void AudioStream::convertPacket(const AkPacket &packet)
 {
     if (!packet)
@@ -253,20 +319,18 @@ void AudioStream::convertPacket(const AkPacket &packet)
     AVFrame iFrame;
     memset(&iFrame, 0, sizeof(AVFrame));
     iFrame.format = codecContext->sample_fmt;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+    av_channel_layout_copy(&iFrame.ch_layout, &codecContext->ch_layout);
+#else
     iFrame.channel_layout = codecContext->channel_layout;
+#endif
     iFrame.sample_rate = codecContext->sample_rate;
-    iFrame.nb_samples = iPacket.caps().samples();
+    iFrame.nb_samples = iPacket.samples();
     iFrame.pts = iPacket.pts();
-    int channels = av_get_channel_layout_nb_channels(iFrame.channel_layout);
 
-    if (av_samples_fill_arrays(iFrame.data,
-                               iFrame.linesize,
-                               reinterpret_cast<const uint8_t *>(iPacket.buffer().constData()),
-                               channels,
-                               iPacket.caps().samples(),
-                               AVSampleFormat(iFrame.format),
-                               1) < 0) {
-        return;
+    for (int plane = 0; plane < iPacket.planes(); ++plane) {
+        iFrame.data[plane] = iPacket.plane(plane);
+        iFrame.linesize[plane] = iPacket.planeSize(plane);
     }
 
     this->d->m_frameMutex.lock();
@@ -274,7 +338,11 @@ void AudioStream::convertPacket(const AkPacket &packet)
     // Create new buffer.
     auto oFrame = av_frame_alloc();
     oFrame->format = codecContext->sample_fmt;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+    av_channel_layout_copy(&oFrame->ch_layout, &codecContext->ch_layout);
+#else
     oFrame->channel_layout = codecContext->channel_layout;
+#endif
     oFrame->sample_rate = codecContext->sample_rate;
     oFrame->nb_samples = (this->d->m_frame? this->d->m_frame->nb_samples: 0)
                          + iFrame.nb_samples;
@@ -287,6 +355,12 @@ void AudioStream::convertPacket(const AkPacket &packet)
         return;
     }
 
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+    int channels = iFrame.ch_layout.nb_channels;
+#else
+    int channels = av_get_channel_layout_nb_channels(iFrame.channel_layout);
+#endif
+
     // Copy old samples to new buffer.
     if (this->d->m_frame)
         if (av_samples_copy(oFrame->data,
@@ -413,11 +487,20 @@ AVFrame *AudioStream::dequeueFrame()
         // Create output buffer.
         oFrame = av_frame_alloc();
         oFrame->format = codecContext->sample_fmt;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+        av_channel_layout_copy(&oFrame->ch_layout, &codecContext->ch_layout);
+#else
         oFrame->channel_layout = codecContext->channel_layout;
+#endif
         oFrame->sample_rate = codecContext->sample_rate;
         oFrame->nb_samples = codecContext->frame_size;
         oFrame->pts = this->d->m_frame->pts;
+
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+        int channels = oFrame->ch_layout.nb_channels;
+#else
         int channels = av_get_channel_layout_nb_channels(oFrame->channel_layout);
+#endif
 
         if (av_frame_get_buffer(oFrame, 0) < 0) {
             this->deleteFrame(&oFrame);
@@ -443,7 +526,11 @@ AVFrame *AudioStream::dequeueFrame()
         // Create new buffer.
         auto frame = av_frame_alloc();
         frame->format = codecContext->sample_fmt;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+        av_channel_layout_copy(&frame->ch_layout, &codecContext->ch_layout);
+#else
         frame->channel_layout = codecContext->channel_layout;
+#endif
         frame->sample_rate = codecContext->sample_rate;
         frame->nb_samples = this->d->m_frame->nb_samples - codecContext->frame_size;
         frame->pts = this->d->m_frame->pts + codecContext->frame_size;
