@@ -26,7 +26,6 @@
 
 #include "multisinkelement.h"
 #include "mediawriter.h"
-#include "multisinkutils.h"
 
 using MediaWriterPtr = QSharedPointer<MediaWriter>;
 
@@ -37,7 +36,6 @@ class MultiSinkElementPrivate
         QString m_location;
         MediaWriterPtr m_mediaWriter;
         QString m_mediaWriterImpl;
-        MultiSinkUtils m_utils;
         QList<int> m_inputStreams;
         QReadWriteLock m_mutex;
 
@@ -47,7 +45,7 @@ class MultiSinkElementPrivate
         QMap<QString, QString> m_formatDescription;
         QStringList m_supportedCodecs;
         QMap<QString, QString> m_codecDescription;
-        QMap<QString, QString> m_codecType;
+        QMap<QString, AkCaps::CapsType> m_codecType;
         QMap<QString, QVariantMap> m_defaultCodecParams;
 
         explicit MultiSinkElementPrivate(MultiSinkElement *self);
@@ -233,7 +231,7 @@ QVariantList MultiSinkElement::formatOptions() const
 }
 
 QStringList MultiSinkElement::supportedCodecs(const QString &format,
-                                              const QString &type)
+                                              AkCaps::CapsType type)
 {
     this->d->m_mutex.lockForRead();
     QStringList supportedCodecs;
@@ -247,7 +245,7 @@ QStringList MultiSinkElement::supportedCodecs(const QString &format,
 }
 
 QString MultiSinkElement::defaultCodec(const QString &format,
-                                       const QString &type)
+                                       AkCaps::CapsType type)
 {
     this->d->m_mutex.lockForRead();
     QString defaultCodec;
@@ -265,7 +263,7 @@ QString MultiSinkElement::codecDescription(const QString &codec) const
     return this->d->m_codecDescription.value(codec);
 }
 
-QString MultiSinkElement::codecType(const QString &codec) const
+AkCaps::CapsType MultiSinkElement::codecType(const QString &codec) const
 {
     return this->d->m_codecType.value(codec);
 }
