@@ -20,6 +20,7 @@
 #ifndef MULTISRCELEMENT_H
 #define MULTISRCELEMENT_H
 
+#include "akcaps.h"
 #include <akmultimediasourceelement.h>
 
 class MultiSrcElementPrivate;
@@ -69,38 +70,38 @@ class MultiSrcElement: public AkMultimediaSourceElement
 
     public:
         enum SeekPosition {
-            SeekSet = 0x0,
-            SeekCur = 0x1,
-            SeekEnd = 0x2,
+            SeekSet,
+            SeekCur,
+            SeekEnd,
         };
-        Q_ENUMS(SeekPosition)
+        Q_ENUM(SeekPosition)
 
         MultiSrcElement();
         ~MultiSrcElement();
 
-        Q_INVOKABLE QStringList medias();
-        Q_INVOKABLE QString media() const;
-        Q_INVOKABLE QList<int> streams();
-        Q_INVOKABLE bool loop() const;
+        Q_INVOKABLE QStringList medias() override;
+        Q_INVOKABLE QString media() const override;
+        Q_INVOKABLE QList<int> streams() override;
+        Q_INVOKABLE bool loop() const override;
         Q_INVOKABLE bool sync() const;
-        Q_INVOKABLE QList<int> listTracks(const QString &type="");
+        Q_INVOKABLE QList<int> listTracks(AkCaps::CapsType type=AkCaps::CapsUnknown);
         Q_INVOKABLE QString streamLanguage(int stream);
-        Q_INVOKABLE int defaultStream(const QString &mimeType);
-        Q_INVOKABLE QString description(const QString &media);
-        Q_INVOKABLE AkCaps caps(int stream);
+        Q_INVOKABLE int defaultStream(AkCaps::CapsType type) override;
+        Q_INVOKABLE QString description(const QString &media) override;
+        Q_INVOKABLE AkCaps caps(int stream) override;
         Q_INVOKABLE qint64 durationMSecs();
         Q_INVOKABLE qint64 currentTimeMSecs();
         Q_INVOKABLE qint64 maxPacketQueueSize() const;
         Q_INVOKABLE bool showLog() const;
-        Q_INVOKABLE ElementState state() const;
+        Q_INVOKABLE AkElement::ElementState state() const override;
 
     private:
         MultiSrcElementPrivate *d;
 
     protected:
-        QString controlInterfaceProvide(const QString &controlId) const;
+        QString controlInterfaceProvide(const QString &controlId) const override;
         void controlInterfaceConfigure(QQmlContext *context,
-                                       const QString &controlId) const;
+                                       const QString &controlId) const override;
 
     signals:
         void mediasChanged(const QStringList &medias);
@@ -116,20 +117,20 @@ class MultiSrcElement: public AkMultimediaSourceElement
         void showLogChanged(bool showLog);
 
     public slots:
-        void seek(qint64 seekTo, SeekPosition position=SeekSet);
-        void setMedia(const QString &media);
-        void setStreams(const QList<int> &streams);
-        void setLoop(bool loop);
+        void seek(qint64 seekTo, MultiSrcElement::SeekPosition position=SeekSet);
+        void setMedia(const QString &media) override;
+        void setStreams(const QList<int> &streams) override;
+        void setLoop(bool loop) override;
         void setSync(bool sync);
         void setMaxPacketQueueSize(qint64 maxPacketQueueSize);
         void setShowLog(bool showLog);
-        void resetMedia();
-        void resetStreams();
-        void resetLoop();
+        void resetMedia() override;
+        void resetStreams() override;
+        void resetLoop() override;
         void resetSync();
         void resetMaxPacketQueueSize();
         void resetShowLog();
-        bool setState(AkElement::ElementState state);
+        bool setState(AkElement::ElementState state) override;
 };
 
 Q_DECLARE_METATYPE(MultiSrcElement::SeekPosition)
