@@ -372,7 +372,7 @@ void MediaSourceVLC::seek(qint64 mSecs,
         break;
     }
 
-    pts = qBound<qint64>(0, pts, duration);
+    pts = qBound(0, qint64(pts), duration);
     libvlc_media_player_set_position(this->d->m_mediaPlayer,
                                      float(pts) / float(duration));
 }
@@ -416,7 +416,7 @@ void MediaSourceVLC::setMedia(const QString &media)
             libvlc_media_release(vlcMedia);
 
             this->d->m_mutex.lock();
-            this->d->m_mediaParsed.wait(&this->d->m_mutex);
+            this->d->m_mediaParsed.wait(&this->d->m_mutex, 5000);
             this->d->m_mutex.unlock();
         } else {
             this->d->m_mutex.lock();
