@@ -22,15 +22,21 @@
 
 #include <akelement.h>
 
+class NormalizeElementPrivate;
+
 class NormalizeElement: public AkElement
 {
     Q_OBJECT
 
     public:
         NormalizeElement();
+        ~NormalizeElement();
+
+    private:
+        NormalizeElementPrivate *d;
 
     protected:
-        AkPacket iVideoStream(const AkVideoPacket &packet);
+        AkPacket iVideoStream(const AkVideoPacket &packet) override;
 };
 
 #endif // NORMALIZEELEMENT_H
