@@ -22,17 +22,15 @@
 
 QObject *Photocopy::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new PhotocopyElement();
-
-    return nullptr;
+    return new PhotocopyElement();
 }
 
 QStringList Photocopy::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_photocopy.cpp"
