@@ -22,17 +22,15 @@
 
 QObject *Swirl::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new SwirlElement();
-
-    return nullptr;
+    return new SwirlElement();
 }
 
 QStringList Swirl::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_swirl.cpp"
