@@ -40,11 +40,21 @@
 #include <strmif.h>
 #include <dbt.h>
 #include <uuids.h>
+#include <wmcodecdsp.h>
 
 #include "capturemmf.h"
 
 #define TIME_BASE 1.0e7
 
+#define AK_DEFINE_GUID(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
+    static const GUID name = {l, w1, w2, {b1, b2, b3, b4, b5, b6, b7, b8}}
+
+AK_DEFINE_GUID(AK_MEDIASUBTYPE_AVC1, 0x31435641, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+AK_DEFINE_GUID(AK_MEDIASUBTYPE_H264, 0x34363248, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+AK_DEFINE_GUID(AK_MEDIASUBTYPE_h264, 0x34363268, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+AK_DEFINE_GUID(AK_MEDIASUBTYPE_X264, 0x34363258, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+AK_DEFINE_GUID(AK_MEDIASUBTYPE_x264, 0x34363278, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+
 Q_CORE_EXPORT HINSTANCE qWinAppInst();
 
 __inline bool operator <(REFGUID guid1, REFGUID guid2)
@@ -105,8 +115,8 @@ inline RawFmtToAkFmtMap initRawFmtToAkFmt()
         {MEDIASUBTYPE_IF09    , AkVideoCaps::Format_yvu410p },
         {MEDIASUBTYPE_IYUV    , AkVideoCaps::Format_yuv420p },
         {MEDIASUBTYPE_NV12    , AkVideoCaps::Format_nv12    },
-        {MEDIASUBTYPE_RGB24   , AkVideoCaps::Format_rgb24   },
-        {MEDIASUBTYPE_RGB32   , AkVideoCaps::Format_0rgbpack},
+        {MEDIASUBTYPE_RGB24   , AkVideoCaps::Format_bgr24   },
+        {MEDIASUBTYPE_RGB32   , AkVideoCaps::Format_xrgbpack},
         {MEDIASUBTYPE_RGB555  , AkVideoCaps::Format_rgb555  },
         {MEDIASUBTYPE_RGB565  , AkVideoCaps::Format_rgb565  },
         {MEDIASUBTYPE_UYVY    , AkVideoCaps::Format_uyvy422 },
@@ -129,23 +139,28 @@ using CompressedFormatToStrMap = QMap<GUID, QString>;
 inline CompressedFormatToStrMap initCompressedFormatToStr()
 {
     CompressedFormatToStrMap compressedFormatToStr {
-        {MEDIASUBTYPE_CFCC  , "mjpg"  },
-        {MEDIASUBTYPE_IJPG  , "jpeg"  },
-        {MEDIASUBTYPE_MDVF  , "dv"    },
-        {MEDIASUBTYPE_MJPG  , "mjpg"  },
-        {MEDIASUBTYPE_Plum  , "mjpg"  },
-        {MEDIASUBTYPE_QTJpeg, "jpeg"  },
-        {MEDIASUBTYPE_QTRle , "qtrle" },
-        {MEDIASUBTYPE_QTRpza, "qtrpza"},
-        {MEDIASUBTYPE_QTSmc , "qtsmc" },
-        {MEDIASUBTYPE_TVMJ  , "mjpg"  },
-        {MEDIASUBTYPE_WAKE  , "mjpg"  },
-        {MEDIASUBTYPE_dv25  , "dv25"  },
-        {MEDIASUBTYPE_dv50  , "dv50"  },
-        {MEDIASUBTYPE_dvh1  , "dvh1"  },
-        {MEDIASUBTYPE_dvhd  , "dvhd"  },
-        {MEDIASUBTYPE_dvsd  , "dvsd"  },
-        {MEDIASUBTYPE_dvsl  , "dvsl"  },
+        {MEDIASUBTYPE_CFCC   , "mjpg"  },
+        {MEDIASUBTYPE_IJPG   , "jpeg"  },
+        {MEDIASUBTYPE_MDVF   , "dv"    },
+        {MEDIASUBTYPE_MJPG   , "mjpg"  },
+        {MEDIASUBTYPE_Plum   , "mjpg"  },
+        {MEDIASUBTYPE_QTJpeg , "jpeg"  },
+        {MEDIASUBTYPE_QTRle  , "qtrle" },
+        {MEDIASUBTYPE_QTRpza , "qtrpza"},
+        {MEDIASUBTYPE_QTSmc  , "qtsmc" },
+        {MEDIASUBTYPE_TVMJ   , "mjpg"  },
+        {MEDIASUBTYPE_WAKE   , "mjpg"  },
+        {MEDIASUBTYPE_dv25   , "dv25"  },
+        {MEDIASUBTYPE_dv50   , "dv50"  },
+        {MEDIASUBTYPE_dvh1   , "dvh1"  },
+        {MEDIASUBTYPE_dvhd   , "dvhd"  },
+        {MEDIASUBTYPE_dvsd   , "dvsd"  },
+        {MEDIASUBTYPE_dvsl   , "dvsl"  },
+        {AK_MEDIASUBTYPE_AVC1, "h264"  },
+        {AK_MEDIASUBTYPE_H264, "h264"  },
+        {AK_MEDIASUBTYPE_h264, "h264"  },
+        {AK_MEDIASUBTYPE_X264, "h264"  },
+        {AK_MEDIASUBTYPE_x264, "h264"  },
     };
 
     return compressedFormatToStr;
@@ -185,6 +200,7 @@ class CaptureMMFPrivate
         QMap<QString, QString> m_descriptions;
         QMap<QString, CaptureVideoCaps> m_devicesCaps;
         qint64 m_id {-1};
+        bool m_hasMediaFoundation {false};
         DWORD m_streamIndex {DWORD(MF_SOURCE_READER_FIRST_VIDEO_STREAM)};
         CaptureMMF::IoMethod m_ioMethod {CaptureMMF::IoMethodSync};
         MediaSourcePtr m_mediaSource;
@@ -236,6 +252,12 @@ CaptureMMF::CaptureMMF(QObject *parent):
     QAbstractNativeEventFilter()
 {
     this->d = new CaptureMMFPrivate(this);
+
+    if (SUCCEEDED(MFStartup(MF_VERSION)))
+        this->d->m_hasMediaFoundation = true;
+    else
+        qCritical() << "Failed initilizing MediaFoundation.";
+
     qApp->installNativeEventFilter(this);
     this->d->updateDevices();
 }
@@ -243,6 +265,10 @@ CaptureMMF::CaptureMMF(QObject *parent):
 CaptureMMF::~CaptureMMF()
 {
     qApp->removeNativeEventFilter(this);
+
+    if (this->d->m_hasMediaFoundation)
+        MFShutdown();
+
     delete this->d;
 }
 
@@ -477,6 +503,17 @@ AkPacket CaptureMMF::readFrame()
     // Read pts.
     LONGLONG sampleTime = 0;
     sample->GetSampleTime(&sampleTime);
+    AkFrac timeBase(1, TIME_BASE);
+
+    if (sampleTime < 1) {
+        AkVideoCaps videoCaps(caps);
+        auto timestamp = QDateTime::currentMSecsSinceEpoch();
+        sampleTime =
+            LONGLONG(qreal(timestamp)
+                     * videoCaps.fps().value()
+                     / 1e3);
+        timeBase = videoCaps.fps().invert();
+    }
 
     if (isRaw) {
         IMF2DBuffer *d2Buffer = nullptr;
@@ -521,7 +558,7 @@ AkPacket CaptureMMF::readFrame()
             d2Buffer->Unlock2D();
 
             packet.setPts(sampleTime);
-            packet.setTimeBase(AkFrac(1, TIME_BASE));
+            packet.setTimeBase(timeBase);
             packet.setIndex(0);
             packet.setId(this->d->m_id);
 
@@ -570,7 +607,7 @@ AkPacket CaptureMMF::readFrame()
         buffer->Unlock();
 
         packet.setPts(sampleTime);
-        packet.setTimeBase(AkFrac(1, TIME_BASE));
+        packet.setTimeBase(timeBase);
         packet.setIndex(0);
         packet.setId(this->d->m_id);
 
@@ -592,7 +629,7 @@ AkPacket CaptureMMF::readFrame()
     buffer->Unlock();
 
     packet.setPts(sampleTime);
-    packet.setTimeBase(AkFrac(1, TIME_BASE));
+    packet.setTimeBase(timeBase);
     packet.setIndex(0);
     packet.setId(this->d->m_id);
 
@@ -603,8 +640,8 @@ AkPacket CaptureMMF::readFrame()
 }
 
 bool CaptureMMF::nativeEventFilter(const QByteArray &eventType,
-                                     void *message,
-                                     long *result)
+                                   void *message,
+                                   qintptr *result)
 {
     Q_UNUSED(eventType)
 
@@ -635,58 +672,101 @@ bool CaptureMMF::nativeEventFilter(const QByteArray &eventType,
 
 bool CaptureMMF::init()
 {
-    if (FAILED(MFStartup(MF_VERSION)))
+    if (!this->d->m_hasMediaFoundation)
         return false;
 
+    qDebug() << "Getting media source for" << this->d->m_device;
     auto mediaSource = this->d->mediaSource(this->d->m_device);
 
-    if (!mediaSource)
-        return false;
+    if (!mediaSource) {
+        qCritical() << "Error getting the media source for" << this->d->m_device;
+        MFShutdown();
 
+        return false;
+    }
+
+    qDebug() << "Creating media source attributes.";
     IMFAttributes *attributes = nullptr;
 
-    if (FAILED(MFCreateAttributes(&attributes, 0)))
-        return false;
+    if (FAILED(MFCreateAttributes(&attributes, 0))) {
+        qCritical() << "Failed to create media source attributes.";
+        MFShutdown();
 
+        return false;
+    }
+
+    qDebug() << "Creating source reader.";
     attributes->SetUINT32(MF_SOURCE_READER_DISCONNECT_MEDIASOURCE_ON_SHUTDOWN,
                           TRUE);
-
-    bool ok = false;
-    auto streams = this->d->m_streams;
-
-    DWORD streamIndex = MF_SOURCE_READER_FIRST_VIDEO_STREAM;
-    DWORD mediaTypeIndex = 0;
     IMFSourceReader *sourceReader = nullptr;
 
     if (FAILED(MFCreateSourceReaderFromMediaSource(mediaSource.data(),
                                                    attributes,
-                                                   &sourceReader)))
-        goto init_failed;
+                                                   &sourceReader))) {
+        qCritical() << "Failed creating the source reader.";
+        attributes->Release();
+        MFShutdown();
 
+        return false;
+    }
+
+    qDebug() << "Configuring the media types.";
     this->d->m_sourceReader =
             SourceReaderPtr(sourceReader,
                             CaptureMMFPrivate::deleteSourceReader);
     this->d->m_sourceReader->SetStreamSelection(MF_SOURCE_READER_ALL_STREAMS,
                                                 FALSE);
 
+    auto streams = this->d->m_streams;
+
     if (this->d->m_streams.isEmpty())
         streams << 0;
 
-    for (auto &stream: streams) {
-        if (!this->d->indexFromTrack(mediaSource.data(),
-                                     stream, &streamIndex, &mediaTypeIndex))
-            goto init_failed;
+    DWORD streamIndex = MF_SOURCE_READER_FIRST_VIDEO_STREAM;
 
+    for (auto &stream: streams) {
+        qDebug() << "Getting index from the selected stream.";
+        DWORD mediaTypeIndex = 0;
+
+        if (!this->d->indexFromTrack(mediaSource.data(),
+                                     stream,
+                                     &streamIndex,
+                                     &mediaTypeIndex)) {
+            qCritical() << "Error getting index from the selected stream.";
+            this->d->m_sourceReader->Flush(MF_SOURCE_READER_ALL_STREAMS);
+            this->d->m_sourceReader.clear();
+            attributes->Release();
+            MFShutdown();
+
+            return false;
+        }
+
+        qDebug() << "Setting the selected stream.";
         this->d->m_sourceReader->SetStreamSelection(streamIndex, TRUE);
         auto mediaStream = this->d->stream(mediaSource.data(), streamIndex);
 
-        if (!mediaStream)
-            goto init_failed;
+        if (!mediaStream) {
+            qCritical() << "Error setting the selected stream.";
+            this->d->m_sourceReader->Flush(MF_SOURCE_READER_ALL_STREAMS);
+            this->d->m_sourceReader.clear();
+            attributes->Release();
+            MFShutdown();
 
+            return false;
+        }
+
+        qDebug() << "Setting the media type.";
         auto mediaType = this->d->mediaType(mediaStream.data(), mediaTypeIndex);
 
-        if (!mediaType)
-            goto init_failed;
+        if (!mediaType) {
+            qCritical() << "Error setting the selected stream.";
+            this->d->m_sourceReader->Flush(MF_SOURCE_READER_ALL_STREAMS);
+            this->d->m_sourceReader.clear();
+            attributes->Release();
+            MFShutdown();
+
+            return false;
+        }
 
         this->d->m_sourceReader->SetCurrentMediaType(streamIndex,
                                                      nullptr,
@@ -702,15 +782,11 @@ bool CaptureMMF::init()
     this->d->m_mediaSource = mediaSource;
     this->d->m_id = Ak::id();
     this->d->m_streamIndex = streamIndex;
-    ok = true;
-
-init_failed:
-    if (!ok)
-        this->uninit();
-
     attributes->Release();
 
-    return ok;
+    qDebug() << "Starting camera capture.";
+
+    return true;
 }
 
 void CaptureMMF::uninit()
@@ -718,7 +794,6 @@ void CaptureMMF::uninit()
     this->d->m_sourceReader->Flush(MF_SOURCE_READER_ALL_STREAMS);
     this->d->m_sourceReader.clear();
     this->d->m_mediaSource.clear();
-    MFShutdown();
 }
 
 void CaptureMMF::setDevice(const QString &device)
@@ -1140,7 +1215,12 @@ AkCaps CaptureMMFPrivate::capsFromMediaType(IMFMediaType *mediaType,
     UINT32 fpsNum = 0;
     UINT32 fpsDen = 0;
     MFGetAttributeRatio(mediaType, MF_MT_FRAME_RATE, &fpsNum, &fpsDen);
-    AkFrac fps(fpsNum, fpsDen);
+    AkFrac fps;
+
+    if (fpsNum > 0 && fpsDen > 0)
+        fps = {fpsNum, fpsDen};
+    else
+        fps = {30, 1};
 
     auto srcLineSize =
             INT32(MFGetAttributeUINT32(mediaType, MF_MT_DEFAULT_STRIDE, 0));
@@ -1458,50 +1538,53 @@ void CaptureMMFPrivate::updateDevices()
     decltype(this->m_devices) devices;
     decltype(this->m_descriptions) descriptions;
     decltype(this->m_devicesCaps) devicesCaps;
-    auto sources = this->sources();
 
-    for (auto &source: sources) {
-        WCHAR *sourceLink = nullptr;
+    if (this->m_hasMediaFoundation) {
+        auto sources = this->sources();
 
-        if (FAILED(source->GetAllocatedString(MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_VIDCAP_SYMBOLIC_LINK,
-                                              &sourceLink,
-                                              nullptr))) {
-            continue;
-        }
+        for (auto &source: sources) {
+            WCHAR *sourceLink = nullptr;
 
-        auto deviceId = QString::fromWCharArray(sourceLink);
-        CoTaskMemFree(sourceLink);
-
-        QString description;
-        WCHAR *friendlyName = nullptr;
-
-        if (SUCCEEDED(source->GetAllocatedString(MF_DEVSOURCE_ATTRIBUTE_FRIENDLY_NAME,
-                                                 &friendlyName,
-                                                 nullptr))) {
-            description = QString::fromWCharArray(friendlyName);
-            CoTaskMemFree(friendlyName);
-        }
-
-        CaptureVideoCaps caps;
-
-        for (auto &stream: this->streams(source))
-            for (auto &mediaType: this->mediaTypes(stream.data())) {
-                auto videoCaps = this->capsFromMediaType(mediaType.data());
-
-                if (videoCaps)
-                    caps << videoCaps;
+            if (FAILED(source->GetAllocatedString(MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_VIDCAP_SYMBOLIC_LINK,
+                                                &sourceLink,
+                                                nullptr))) {
+                continue;
             }
 
-        if (!caps.isEmpty()) {
-            devices << deviceId;
-            descriptions[deviceId] = description;
-            devicesCaps[deviceId] = caps;
-        }
-    }
+            auto deviceId = QString::fromWCharArray(sourceLink);
+            CoTaskMemFree(sourceLink);
 
-    if (devicesCaps.isEmpty()) {
-        devices.clear();
-        descriptions.clear();
+            QString description;
+            WCHAR *friendlyName = nullptr;
+
+            if (SUCCEEDED(source->GetAllocatedString(MF_DEVSOURCE_ATTRIBUTE_FRIENDLY_NAME,
+                                                    &friendlyName,
+                                                    nullptr))) {
+                description = QString::fromWCharArray(friendlyName);
+                CoTaskMemFree(friendlyName);
+            }
+
+            CaptureVideoCaps caps;
+
+            for (auto &stream: this->streams(source))
+                for (auto &mediaType: this->mediaTypes(stream.data())) {
+                    auto videoCaps = this->capsFromMediaType(mediaType.data());
+
+                    if (videoCaps)
+                        caps << videoCaps;
+                }
+
+            if (!caps.isEmpty()) {
+                devices << deviceId;
+                descriptions[deviceId] = description;
+                devicesCaps[deviceId] = caps;
+            }
+        }
+
+        if (devicesCaps.isEmpty()) {
+            devices.clear();
+            descriptions.clear();
+        }
     }
 
     this->m_descriptions = descriptions;
