@@ -26,7 +26,7 @@ class VirtualCamera: public QObject, public AkPlugin
 {
     Q_OBJECT
     Q_INTERFACES(AkPlugin)
-    Q_PLUGIN_METADATA(IID "org.avkys.plugin" FILE "../pspec.json")
+    Q_PLUGIN_METADATA(IID AkPlugin_IID FILE "pspec.json")
 
     public:
         VirtualCamera();
