@@ -518,6 +518,18 @@ QStringList VirtualCameraElement::availableRootMethods() const
     return methods;
 }
 
+bool VirtualCameraElement::canEditVCamDescription() const
+{
+    this->d->m_mutex.lock();
+    auto vcam = this->d->m_vcam;
+    this->d->m_mutex.unlock();
+
+    if (vcam)
+        return vcam->canEditVCamDescription();
+
+    return false;
+}
+
 QString VirtualCameraElement::controlInterfaceProvide(const QString &controlId) const
 {
     Q_UNUSED(controlId)
