@@ -22,17 +22,15 @@
 
 QObject *Warp::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new WarpElement();
-
-    return nullptr;
+    return new WarpElement();
 }
 
 QStringList Warp::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_warp.cpp"
