@@ -32,29 +32,37 @@ class WarpElement: public AkElement
                WRITE setRipples
                RESET resetRipples
                NOTIFY ripplesChanged)
+    Q_PROPERTY(qreal duration
+               READ duration
+               WRITE setDuration
+               RESET resetDuration
+               NOTIFY durationChanged)
 
     public:
         WarpElement();
         ~WarpElement();
 
         Q_INVOKABLE qreal ripples() const;
+        Q_INVOKABLE int duration() const;
 
     private:
         WarpElementPrivate *d;
 
     protected:
-        QString controlInterfaceProvide(const QString &controlId) const;
+        QString controlInterfaceProvide(const QString &controlId) const override;
         void controlInterfaceConfigure(QQmlContext *context,
-                                       const QString &controlId) const;
-        AkPacket iVideoStream(const AkVideoPacket &packet);
+                                       const QString &controlId) const override;
+        AkPacket iVideoStream(const AkVideoPacket &packet) override;
 
     signals:
         void ripplesChanged(qreal ripples);
-        void frameSizeChanged(const QSize &frameSize);
+        void durationChanged(int duration);
 
     public slots:
         void setRipples(qreal ripples);
+        void setDuration(int duration);
         void resetRipples();
+        void resetDuration();
 };
 
 #endif // WARPELEMENT_H
