@@ -22,17 +22,15 @@
 
 QObject *Wave::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new WaveElement();
-
-    return nullptr;
+    return new WaveElement();
 }
 
 QStringList Wave::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_wave.cpp"
